@@ -21,34 +21,26 @@
  */
 
 #include <config.h>
+#include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
 #include <unistd.h>
-
 #include <gtk/gtk.h>
-#include <libgnome/libgnome.h>
-#include <libgnomeui/gnome-help.h>
-#include <libgnomeui/gnome-ditem-edit.h>
-#include <libgnomeui/gnome-window-icon.h>
-#include <libgnomevfs/gnome-vfs-ops.h>
-#include <glade/glade.h>
-
+#include "dlg-properties.h"
+#include "dlg-add-from-menu.h"
 #include "file-utils.h"
 #include "gtk-utils.h"
 #include "quick-lounge.h"
 #include "quick-box.h"
 #include "quick-button.h"
 #include "quick-separator.h"
-#include "dlg-properties.h"
-#include "dlg-pick-applications.h"
 
-
-#define GLADE_FILE             "quick-lounge.glade"
-#define SEPARATOR_NAME         _("Space")
+#define SEPARATOR_NAME         "---" /*_("Space")*/
 #define ICON_SIZE              24
 #define UPDATE_MAX_SIZE_DELAY  100
 #define AUTO_SCROLL_STEP 4
 #define AUTO_SCROLL_THRESHOLD 20
+#define GET_WIDGET(name) _gtk_builder_get_widget (data->builder, (name))
 
 
 enum {
@@ -83,8 +75,8 @@ typedef struct {
 	QuickBox      *quick_box;
 	GtkWidget     *select_button;
 
-	GladeXML      *gui;
-
+	GtkBuilder    *builder;
+	 
 	GtkWidget     *dialog;
 	GtkWidget     *tree_view;
 	GtkWidget     *btn_up;
@@ -100,7 +92,7 @@ typedef struct {
 
 	GtkWidget     *p_minsize_spinbutton;
 	GtkWidget     *p_maxsize_spinbutton;
-	GtkWidget     *p_icon_size_checkbutton;
+	GtkWidget     *p_rows_spinbutton;
 	GtkWidget     *p_apps_scrolledwindow;
 	GtkAdjustment *p_apps_vadjustment;
 
@@ -116,13 +108,16 @@ static gboolean
 change_max_size (gpointer user_data)
 {
 	DialogData *data = user_data;
-
+	gboolean    needs_update;
+	
 	g_source_remove (data->timeout);
 	data->timeout = 0;
 
-	quick_lounge_set_max_size (data->quick_lounge, gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (data->p_maxsize_spinbutton)));
+	needs_update = quick_lounge_set_max_size (data->quick_lounge, gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (data->p_maxsize_spinbutton)));
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (data->p_maxsize_spinbutton), quick_box_get_max_visible_cols (data->quick_box));
-
+	if (needs_update)
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (data->p_minsize_spinbutton), quick_box_get_min_visible_cols (data->quick_box));
+		
 	return FALSE;
 }
 
@@ -134,8 +129,7 @@ destroy_cb (GtkWidget  *widget,
 {
 	if (data->timeout != 0) 
 		change_max_size (data);
-
-	g_object_unref (data->gui);
+	g_object_unref (data->builder);
 	g_free (data);
 }
 
@@ -154,39 +148,7 @@ static void
 help_cb (GtkWidget  *widget, 
 	 DialogData *data)
 {
-	GError *err;
-
-	err = NULL;  
-	gnome_help_display_desktop_on_screen (
-	      NULL, 
-	      "quick-lounge", 
-	      "quick-lounge", 
-	      "quick-lounge-prefs", 
-	      gtk_widget_get_screen (GTK_WIDGET (data->quick_lounge->applet)),
-	      &err); 
-	
-	if (err != NULL) {
-		GtkWidget *dialog;
-		
-		dialog = gtk_message_dialog_new (GTK_WINDOW (data->dialog),
-						 0,
-						 GTK_MESSAGE_ERROR,
-						 GTK_BUTTONS_CLOSE,
-						 _("Could not display help: %s"),
-						 err->message);
-		gtk_window_set_icon_name (GTK_WINDOW (dialog), "quick-lounge-applet");
-		gtk_window_set_screen (GTK_WINDOW (dialog), gtk_widget_get_screen (GTK_WIDGET (data->quick_lounge->applet)));
-		
-		g_signal_connect (G_OBJECT (dialog), "response",
-				  G_CALLBACK (gtk_widget_destroy),
-				  NULL);
-		
-		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
-		
-		gtk_widget_show (dialog);
-		
-		g_error_free (err);
-	}
+	_gtk_show_help (GTK_WINDOW (data->dialog), "quick-lounge", "quick-lounge-prefs");
 }
 
 
@@ -292,7 +254,11 @@ move_up_cb (GtkWidget  *widget,
 	if (! gtk_tree_selection_get_selected (selection, NULL, &iter))
 		return;
 
-	position = get_iter_position (data, &iter) - 1;
+	position = get_iter_position (data, &iter);
+	if (position == 0)
+		return;
+	position--;
+	
 	button = tree_view_move_iter_to (data, &iter, position, FALSE);
 	select_and_view_iter (data, &iter);
 	quick_box_reorder_child (data->quick_box, button, position);
@@ -301,7 +267,7 @@ move_up_cb (GtkWidget  *widget,
 }
 
 
-static gboolean
+static void
 move_down_cb (GtkWidget  *widget, 
 	      DialogData *data)
 {
@@ -312,18 +278,20 @@ move_down_cb (GtkWidget  *widget,
 
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (data->tree_view));
 	if (selection == NULL)
-                return FALSE;
+                return;
 	if (! gtk_tree_selection_get_selected (selection, NULL, &iter))
-		return FALSE;
+		return;
 
-	position = get_iter_position (data, &iter) + 1;
+	position = get_iter_position (data, &iter);
+	if (position >= data->n - 1)
+		return;
+	position++;
+	
 	button = tree_view_move_iter_to (data, &iter, position, FALSE);
 	select_and_view_iter (data, &iter);
 	quick_box_reorder_child (data->quick_box, button, position);
 	
 	quick_lounge_save_order (data->quick_lounge);
-	
-	return FALSE;
 }
 
 
@@ -343,6 +311,9 @@ move_bottom_cb (GtkWidget  *widget,
 		return;
 
 	position = get_iter_position (data, &iter);
+	if (position >= data->n - 1)
+		return;
+	
 	button = tree_view_move_iter_to (data, &iter, data->n - 2, TRUE);
 	select_and_view_iter (data, &iter);
 	quick_box_reorder_child (data->quick_box, button, data->n - 1);
@@ -367,6 +338,9 @@ move_top_cb (GtkWidget  *widget,
 		return;
 
 	position = get_iter_position (data, &iter);
+	if (position == 0)
+		return;
+		
 	button = tree_view_move_iter_to (data, &iter, 0, FALSE);
 	select_and_view_iter (data, &iter);
 	quick_box_reorder_child (data->quick_box, button, 0);
@@ -381,28 +355,33 @@ update_sensitivity (DialogData *data)
 	GtkTreeSelection *selection;
 	GtkTreeIter       iter;
 	GtkWidget        *button;
-
+	int               position;
+	
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (data->tree_view));
 	if (selection == NULL)
                 return;
 	
 	if (! gtk_tree_selection_get_selected (selection, NULL, &iter)
-	    || (data->n == 0)) {
+	    || (data->n == 0)) 
+	{
 		gtk_widget_set_sensitive (data->btn_edit, FALSE);
 		gtk_widget_set_sensitive (data->btn_delete, FALSE);
-
 		return;
 	}
 
-	gtk_widget_set_sensitive (data->btn_edit, TRUE);
 	gtk_widget_set_sensitive (data->btn_delete, TRUE);
+
+	position = get_iter_position (data, &iter);
+	gtk_widget_set_sensitive (data->btn_top, position > 0);
+	gtk_widget_set_sensitive (data->btn_up, position > 0);
+	gtk_widget_set_sensitive (data->btn_down, position < data->n - 1);	
+	gtk_widget_set_sensitive (data->btn_bottom, position < data->n - 1);
 	
 	gtk_tree_model_get (data->model,
 			    &iter,
 			    COLUMN_DATA, &button,
 			    -1);
-	gtk_widget_set_sensitive (data->btn_edit, 
-				  ! QUICK_IS_SEPARATOR (button));
+	gtk_widget_set_sensitive (data->btn_edit, ! QUICK_IS_SEPARATOR (button));
 }
 
 
@@ -418,9 +397,8 @@ selection_changed_cb (GtkTreeSelection *selection,
 static void
 update_list (DialogData *data)
 {
-	GList        *children, *scan;
-	GtkBox       *box = GTK_BOX (data->quick_box);
-	GtkTreeIter   select_iter;
+	GList       *children, *scan;
+	GtkTreeIter  select_iter;
 
 	gtk_list_store_clear (GTK_LIST_STORE (data->model));
 
@@ -477,7 +455,7 @@ update_list (DialogData *data)
 		gtk_list_store_set (GTK_LIST_STORE (data->model), &iter,
 				    COLUMN_DATA, button,
 				    COLUMN_ICON, pixbuf,
-				    COLUMN_NAME, button->text,
+				    COLUMN_NAME, button->name,
 				    -1);
 		g_object_unref (pixbuf);
 	}
@@ -558,7 +536,7 @@ add_from_menu_cb (GtkWidget  *widget,
 	else
 		pos = get_iter_position (data, &iter);
 
-	dlg_pick_applications (data->quick_lounge, pos);
+	dlg_add_from_menu (data->quick_lounge, pos);
 }
 
 
@@ -638,8 +616,13 @@ delete_cb (GtkWidget  *widget,
 			    -1);
 
 	uri = g_object_get_data (G_OBJECT (button), "uri");
-	if (uri != NULL)
-		gnome_vfs_unlink (uri);
+	if (uri != NULL) {
+		GFile *file;
+		
+		file = g_file_new_for_uri (uri);
+		g_file_delete (file, NULL, NULL);
+		g_object_unref (file);
+	}
 
 	gtk_container_remove (GTK_CONTAINER (data->quick_box), button);
 	gtk_list_store_remove (GTK_LIST_STORE (data->model), &iter);
@@ -656,7 +639,6 @@ edit_cb (GtkWidget  *widget,
 	GtkTreeSelection *selection;
 	GtkTreeIter       iter;
 	GtkWidget        *button;
-	GnomeDesktopItem *ditem;
 
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (data->tree_view));
 	if (selection == NULL)
@@ -670,11 +652,8 @@ edit_cb (GtkWidget  *widget,
 			    COLUMN_DATA, &button,
 			    -1);
 
-	ditem = g_object_get_data (G_OBJECT (button), "ditem");
-	if (ditem == NULL)
-		return;
-
-	quick_lounge_util__item_properties (data->quick_lounge, button);
+	if (g_object_get_data (G_OBJECT (button), "desktop_entry") != NULL)
+		quick_lounge_util__item_properties (data->quick_lounge, button);
 }
 
 
@@ -682,8 +661,12 @@ static void
 min_size_value_changed_cb (GtkWidget  *widget,
 			   DialogData *data)
 {
-	quick_lounge_set_min_visible_cols (data->quick_lounge, gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (data->p_minsize_spinbutton)));
+	gboolean needs_update;
+	
+	needs_update = quick_lounge_set_min_visible_cols (data->quick_lounge, gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (data->p_minsize_spinbutton)));	 
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (data->p_minsize_spinbutton), quick_box_get_min_visible_cols (data->quick_box));
+	if (needs_update)
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (data->p_maxsize_spinbutton), quick_box_get_max_visible_cols(data->quick_box));
 }
 
 
@@ -698,61 +681,10 @@ max_size_value_changed_cb (GtkWidget  *widget,
 
 
 static void
-icon_size_toggled_cb (GtkWidget  *widget,
-		      DialogData *data)
+rows_value_changed_cb (GtkWidget  *widget,
+		       DialogData *data)
 {
-	quick_lounge_set_icon_size_follows_panel (data->quick_lounge, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->p_icon_size_checkbutton)));
-}
-
-
-static GtkWidget *
-get_button_from_uri (QuickBox   *quick_box,
-		     const char *uri1)
-{
-	GtkBox    *box = GTK_BOX (quick_box);
-	GList     *children, *scan;
-	GtkWidget *retval = NULL;
-
-	children = gtk_container_get_children (GTK_CONTAINER (quick_box));
-	for (scan = children; scan; scan = scan->next) {
-		GtkWidget   *child_widget = scan->data;
-		const char  *uri2;
-
-		uri2 = g_object_get_data (G_OBJECT (child_widget), "uri");
-		if (uri2 == NULL)
-			continue;
-		if (strcmp (uri1, uri2) == 0) {
-			retval = child_widget;
-			break;
-		}
-	}
-	g_list_free (children);
-	
-	return retval;
-}
-
-
-static int
-get_child_position (QuickBox  *quick_box,
-		    GtkWidget *widget)
-{
-	GList    *children, *scan;
-	gboolean  found = FALSE;
-	int       pos;
-	
-	children = gtk_container_get_children (GTK_CONTAINER (quick_box));
-	for (pos = 0, scan = children; scan; pos++, scan = scan->next) {
-		GtkWidget *child_widget = scan->data;
-		if (child_widget == widget) {
-			found = TRUE;
-			break;
-		}
-	}
-
-	if (found)
-		return pos;
-	else
-		return -1;
+	quick_lounge_set_rows (data->quick_lounge, gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (data->p_rows_spinbutton)));
 }
 
 
@@ -814,8 +746,8 @@ drag_data_received  (GtkWidget          *widget,
 		
 		button = tree_view_move_iter_to (ddata, &iter, pos, FALSE);
 		quick_box_reorder_child (ddata->quick_box, button, pos);
-
-	} else {
+	} 
+	else {
 		list = get_file_list_from_url_list (selection_data);
 		for (scan = list; scan; scan = scan->next) {
 			char      *src_uri = scan->data;
@@ -847,13 +779,13 @@ drag_data_received  (GtkWidget          *widget,
 			 * the laucher has been dragged from the properties 
 			 * dialog: change launcher position. */
 
-			button = get_button_from_uri (quick_box, uri);
+			button = quick_box_get_child_from_uri (quick_box, uri);
 			g_free (uri);
 				
 			if (button == NULL) 
 				continue;
 				
-			button_pos = get_child_position (quick_box, button);
+			button_pos = quick_box_get_child_position (quick_box, button);
 			if (button_pos == -1) 
 				continue;
 			
@@ -910,8 +842,8 @@ drag_motion (GtkWidget          *widget,
 	if ((widget->allocation.height - y < AUTO_SCROLL_THRESHOLD)
 	    && (adj->value + widget->allocation.height < adj->upper)) {
 		gtk_adjustment_set_value (adj, gtk_adjustment_get_value (adj) + AUTO_SCROLL_STEP);
-
-	} else 	if ((y < AUTO_SCROLL_THRESHOLD) && (adj->value > 0.0)) {
+	} 
+	else 	if ((y < AUTO_SCROLL_THRESHOLD) && (adj->value > 0.0)) {
 		gtk_adjustment_set_value (adj, gtk_adjustment_get_value (adj) - AUTO_SCROLL_STEP);
 	}
 
@@ -1024,7 +956,8 @@ drag_data_get  (GtkWidget        *widget,
 					8, 
 					(guchar*) pos_data,
 					strlen (pos_data));
-	} else 
+	} 
+	else 
 		gtk_selection_data_set (selection_data, 
 					selection_data->target,
 					8, 
@@ -1033,9 +966,6 @@ drag_data_get  (GtkWidget        *widget,
 }
 
 
-
-
-
 GtkWidget *
 dlg_properties (QuickLounge  *quick_lounge,
 		GtkWidget    *select_button)
@@ -1044,7 +974,9 @@ dlg_properties (QuickLounge  *quick_lounge,
 	GtkWidget        *btn_close;
 	GtkWidget        *btn_help;
 	GtkTreeSelection *selection;
-
+	char             *rows_label;
+	char             *columns_label;
+	
 	data = g_new0 (DialogData, 1);
 
 	data->quick_lounge = quick_lounge;
@@ -1052,37 +984,32 @@ dlg_properties (QuickLounge  *quick_lounge,
 	data->select_button = select_button;
 	data->timeout = 0;
 
-	data->gui = glade_xml_new (GLADEDIR "/" GLADE_FILE , NULL, NULL);
-        if (! data->gui) {
-		g_free (data);
-                g_warning ("Could not find " GLADE_FILE "\n");
-                return NULL;
-        }
+	data->builder = _gtk_builder_new_from_file ("properties.ui");
 
 	/* Get the widgets. */
 
-	data->dialog = glade_xml_get_widget (data->gui, "properties_dialog");
-	data->tree_view = glade_xml_get_widget (data->gui, "p_apps_treeview");
-	data->btn_up = glade_xml_get_widget (data->gui, "p_up_button");
-	data->btn_down = glade_xml_get_widget (data->gui, "p_down_button");
-	data->btn_top = glade_xml_get_widget (data->gui, "p_top_button");
-	data->btn_bottom = glade_xml_get_widget (data->gui, "p_bottom_button");
+	data->dialog = GET_WIDGET ("properties_dialog");
+	data->tree_view = GET_WIDGET ("p_apps_treeview");
+	data->btn_up = GET_WIDGET ("p_up_button");
+	data->btn_down = GET_WIDGET ("p_down_button");
+	data->btn_top = GET_WIDGET ("p_top_button");
+	data->btn_bottom = GET_WIDGET ("p_bottom_button");
 
-	data->btn_edit = glade_xml_get_widget (data->gui, "p_edit_button");
-	data->btn_add_launcher = glade_xml_get_widget (data->gui, "p_add_launcher_button");
-	data->btn_add_from_menu = glade_xml_get_widget (data->gui, "p_add_from_menu_button");
-	data->btn_add_space = glade_xml_get_widget (data->gui, "p_add_space_button");
-	data->btn_delete = glade_xml_get_widget (data->gui, "p_delete_button");
+	data->btn_edit = GET_WIDGET ("p_edit_button");
+	data->btn_add_launcher = GET_WIDGET ("p_add_launcher_button");
+	data->btn_add_from_menu = GET_WIDGET ("p_add_from_menu_button");
+	data->btn_add_space = GET_WIDGET ("p_add_space_button");
+	data->btn_delete = GET_WIDGET ("p_delete_button");
 
-	data->p_minsize_spinbutton = glade_xml_get_widget (data->gui, "p_minsize_spinbutton");
-	data->p_maxsize_spinbutton = glade_xml_get_widget (data->gui, "p_maxsize_spinbutton");
-	data->p_icon_size_checkbutton = glade_xml_get_widget (data->gui, "p_icon_size_checkbutton");
-	data->p_apps_scrolledwindow = glade_xml_get_widget (data->gui, "p_apps_scrolledwindow");
+	data->p_minsize_spinbutton = GET_WIDGET ("p_minsize_spinbutton");
+	data->p_maxsize_spinbutton = GET_WIDGET ("p_maxsize_spinbutton");
+	data->p_rows_spinbutton = GET_WIDGET ("p_rows_spinbutton");
 
+	data->p_apps_scrolledwindow = GET_WIDGET ("p_apps_scrolledwindow");
 	data->p_apps_vadjustment = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (data->p_apps_scrolledwindow));
 
-	btn_close = glade_xml_get_widget (data->gui, "p_close_button");
-	btn_help = glade_xml_get_widget (data->gui, "p_help_button");
+	btn_close = GET_WIDGET ("p_close_button");
+	btn_help = GET_WIDGET ("p_help_button");
 
 	/* Set widgets data. */
 
@@ -1100,11 +1027,27 @@ dlg_properties (QuickLounge  *quick_lounge,
 	gtk_widget_set_sensitive (data->btn_top, TRUE);
 	gtk_widget_set_sensitive (data->btn_bottom, TRUE);
 
+	switch (quick_box_get_orient (data->quick_box)) {
+	case PANEL_APPLET_ORIENT_LEFT:
+	case PANEL_APPLET_ORIENT_RIGHT:
+		rows_label = _("C_olumns:");
+		columns_label = _("rows");
+		break;
+	case PANEL_APPLET_ORIENT_UP:
+	case PANEL_APPLET_ORIENT_DOWN:
+	default:
+		rows_label = _("R_ows:");
+		columns_label = _("columns");
+		break;
+	}
+	gtk_label_set_text_with_mnemonic (GTK_LABEL (GET_WIDGET ("rows_label")), rows_label);
+	gtk_label_set_text (GTK_LABEL (GET_WIDGET ("columns_label")), columns_label);
+
 	/**/
 
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (data->p_minsize_spinbutton), quick_box_get_min_visible_cols (data->quick_box));
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (data->p_maxsize_spinbutton), quick_box_get_max_visible_cols (data->quick_box));
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->p_icon_size_checkbutton), quick_box_get_icon_size_follows_panel (data->quick_box));
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON (data->p_rows_spinbutton), quick_box_get_rows (data->quick_box));
 
 	/* Set the signals handlers. */
 	
@@ -1172,11 +1115,11 @@ dlg_properties (QuickLounge  *quick_lounge,
                           "value_changed",
                           G_CALLBACK (max_size_value_changed_cb),
 			  data);
-	g_signal_connect (G_OBJECT (data->p_icon_size_checkbutton),
-			  "toggled",
-                          G_CALLBACK (icon_size_toggled_cb),
+	g_signal_connect (G_OBJECT (data->p_rows_spinbutton),
+			  "value_changed",
+			  G_CALLBACK (rows_value_changed_cb),
 			  data);
-
+	
 	/* Drag & Drop */
 
 	gtk_drag_source_set (data->tree_view,
